==> torch_mist-0.2.5.tar.gz: torch_mist-0.2.5/src/torch_mist/utils/train/utils.py <==
import os.path
import tempfile
from copy import deepcopy
from typing import Optional

from torch import nn
from wandb.wandb_torch import torch


class RunTerminationManager:
    def __init__(
        self,
        early_stopping: bool,
        tolerance: float,
        patience: int,
        verbose: bool,
        warmup_iterations: Optional[int],
        max_iterations: Optional[int],
        maximize: bool = False,
        minimize: bool = False,
    ):
        if early_stopping:
            if not maximize and not minimize:
                print(
                    "[Warning]: early_stopping can be used only when maximizing or minimizing, the parameter will be ignored."
                )
                early_stopping = False
        self.early_stopping = early_stopping
        self.maximize = maximize
        self.minimize = minimize
        self.best_value = 0
        self.tolerance = tolerance
        self.patience = patience
        self.current_patience = patience
        self.verbose = verbose
        self.max_iterations = max_iterations
        self.warmup_iterations = warmup_iterations
        self.best_state_dict = None
        self.__best_model_path = None

    def save_weights(self, model: nn.Module, iteration: int):
        self.delete_best_weights()
        self.__best_model_path = os.path.join(tempfile.gettempdir(), f'model_{iteration}.pyt')
        torch.save(model.state_dict(), self.__best_model_path)

    def load_best_weights(self, model: nn.Module):
        if not (self.__best_model_path is None):
            if self.verbose:
                iteration = self.__best_model_path.split('_')[-1].split('.')[0]
                print(f'Loading the weights saved at iteration {iteration}')
            model.load_state_dict(torch.load(self.__best_model_path))
        else:
            if self.verbose:
                print(f'Using the weights from the last iteration')

    def delete_best_weights(self):
        if not (self.__best_model_path is None):
            os.remove(self.__best_model_path)

    def should_stop(
        self, iteration: int, score: Optional[float], model: nn.Module
    ) -> bool:
        stop = False

        if iteration <= self.warmup_iterations:
            return False

        if self.early_stopping:
            improvement = (
                (score - self.best_value)
                if self.maximize
                else (self.best_value - score)
            )
            if improvement > 0:
                # Improvement, update best and reset the patience
                self.best_value = score
                self.current_patience = self.patience

                # Write the best state_dict to disk
                self.save_weights(model, iteration)

                self.best_state_dict = deepcopy(model.state_dict())
                if self.verbose:
                    print(f"Best value: {score}")
            elif -improvement < self.tolerance:
                pass
            else:
                self.current_patience -= 1
                if self.verbose:
                    print(f"Losing patience: {self.current_patience}")

            if self.current_patience <= 0:
                if self.verbose:
                    print("No improvements on validation, stopping.")
                stop = True

        if self.max_iterations:
            if iteration >= self.max_iterations:
                stop = True
        return stop

    def __del__(self):
        self.delete_best_weights()

==> torch_mist-0.2.10.tar.gz: torch_mist-0.2.10/src/torch_mist/utils/train/utils.py <==
import os.path
import tempfile
from copy import deepcopy
from typing import Optional

from torch import nn
import torch


class RunTerminationManager:
    def __init__(
        self,
        early_stopping: bool,
        tolerance: float,
        patience: int,
        verbose: bool,
        warmup_iterations: Optional[int],
        max_iterations: Optional[int],
        maximize: bool = False,
        minimize: bool = False,
    ):
        if early_stopping:
            if not maximize and not minimize:
                print(
                    "[Warning]: early_stopping can be used only when maximizing or minimizing, the parameter will be ignored."
                )
                early_stopping = False
        self.early_stopping = early_stopping
        self.maximize = maximize
        self.minimize = minimize
        self.best_value = 0
        self.tolerance = tolerance
        self.patience = patience
        self.current_patience = patience
        self.verbose = verbose
        self.max_iterations = max_iterations
        self.warmup_iterations = warmup_iterations
        self.best_state_dict = None
        self.__best_model_path = None

    def save_weights(self, model: nn.Module, iteration: int):
        self.delete_best_weights()
        self.__best_model_path = os.path.join(
            tempfile.gettempdir(), f"model_{iteration}.pyt"
        )
        torch.save(model.state_dict(), self.__best_model_path)

    def load_best_weights(self, model: nn.Module):
        if not (self.__best_model_path is None):
            if self.verbose:
                iteration = self.__best_model_path.split("_")[-1].split(".")[0]
                print(f"Loading the weights saved at iteration {iteration}")
            model.load_state_dict(torch.load(self.__best_model_path))
        else:
            if self.verbose:
                print(f"Using the weights from the last iteration")

    def delete_best_weights(self):
        if not (self.__best_model_path is None):
            os.remove(self.__best_model_path)

    def should_stop(
        self, iteration: int, score: Optional[float], model: nn.Module
    ) -> bool:
        stop = False

        if iteration <= self.warmup_iterations:
            return False

        if self.early_stopping:
            improvement = (
                (score - self.best_value)
                if self.maximize
                else (self.best_value - score)
            )
            if improvement > 0:
                # Improvement, update best and reset the patience
                self.best_value = score
                self.current_patience = self.patience

                # Write the best state_dict to disk
                self.save_weights(model, iteration)

                self.best_state_dict = deepcopy(model.state_dict())
                if self.verbose:
                    print(f"Best value: {score}")
            elif -improvement < self.tolerance:
                pass
            else:
                self.current_patience -= 1
                if self.verbose:
                    print(f"Losing patience: {self.current_patience}")

            if self.current_patience <= 0:
                if self.verbose:
                    print("No improvements on validation, stopping.")
                stop = True

        if self.max_iterations:
            if iteration >= self.max_iterations:
                stop = True
        return stop

    def __del__(self):
        self.delete_best_weights()
